﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Linq;
using System.Windows.Controls;
using Telerik.Pivot.Core;
using Telerik.Pivot.Core.Fields;

namespace CustomDateTimeGroupDescription
{
    /// <summary>
    /// Interaction logic for CustomDateTimeGroup.xaml
    /// </summary>
    public partial class CustomDateTimeGroup : UserControl
    {
        public CustomDateTimeGroup()
        {
            InitializeComponent();
            var provider = this.Resources["dataProvider"] as LocalDataSourceProvider;

            provider.PrepareDescriptionForField += provider_PrepareDescription;
        }

        private void provider_PrepareDescription(object sender, PrepareDescriptionForFieldEventArgs e)
        {
            if (e.DescriptionType == DataProviderDescriptionType.Group && e.FieldInfo.DataType == typeof(DateTime))
            {
                e.Description = new MyDateTimeGroupDescription() { PropertyName = e.FieldInfo.Name };
            }
        }
    }

    public class MyFieldDescriptionProvider : LocalDataSourceFieldDescriptionsProvider
    {
        protected override ContainerNode GetFieldDescriptionHierarchy(IEnumerable<IPivotFieldInfo> descriptions)
        {
            var root = new ContainerNode("Root", ContainerNodeRole.None);
            foreach (var fieldInfoItem in descriptions)
            {
                var fieldDescriptionNode = new FieldInfoNode(fieldInfoItem);
                root.Children.Add(fieldDescriptionNode);
            }
            return root;
        }
    }

    public class MyDateTimeGroupDescription : PropertyGroupDescriptionBase
    {
        protected override void CloneOverride(Cloneable source)
        {
            // Copy all properties from source to 'this' description is applicable.
        }

        protected override Cloneable CreateInstanceCore()
        {
            return new MyDateTimeGroupDescription();
        }

        protected override object GroupNameFromItem(object item, int level)
        {
            var baseValue = base.GroupNameFromItem(item, level);

            if (baseValue == null)
            {
                return null;
            }
            else
            {
                return new DayMonthYearGroup(((DateTime)baseValue).Year, ((DateTime)baseValue).Month, ((DateTime)baseValue).Day, this.Culture);
            }
        }
    }

    public struct DayMonthYearGroup : IComparable, IComparable<DayMonthYearGroup>, IEquatable<DayMonthYearGroup>
    {
        public DayMonthYearGroup(int year, int month, int day, CultureInfo culture)
            : this()
        {
            this.Year = year;
            this.Month = month;
            this.Day = day;
            this.Culture = culture;
        }

        public int Day
        {
            get;
            private set;
        }
        
        public int Year
        {
            get;
            private set;
        }

        public int Month
        {
            get;
            private set;
        }

        public CultureInfo Culture
        {
            get;
            private set;
        }
        
        public override string ToString()
        {
            return new DateTime(this.Year, this.Month, this.Day).ToString("d", this.Culture);
        }
        
        public override int GetHashCode()
        {
            return (this.Year * this.Month * this.Day) ^ 211;
        }
                
        public override bool Equals(object obj)
        {
            if (obj is DayMonthYearGroup)
            {
                return this.Equals((DayMonthYearGroup)obj);
            }

            return false;
        }

        public bool Equals(DayMonthYearGroup other)
        {
            return (this.Year == other.Year && this.Month == other.Month && this.Day == other.Day);
        }
        
        public int CompareTo(object obj)
        {
            if (obj is DayMonthYearGroup)
            {
                return this.CompareTo((DayMonthYearGroup)obj);
            }

            throw new ArgumentException("Can not compare.", "obj");
        }
        
        public int CompareTo(DayMonthYearGroup other)
        {
            var yearCompare = this.Year.CompareTo(other.Year);
            var monthCompare = yearCompare == 0 ? this.Month.CompareTo(other.Month) : yearCompare;
            return monthCompare == 0 ? this.Day.CompareTo(other.Day) : monthCompare;
        }
    }
}
